/*  CALLDEFS.H - Standard Definitions for User-added Callable Routines

    Copyright (C) 1994, 1998-2000, 2004, 2008, 2011 Envyr Corporation,
    All Rights Reserved.

    Revision:
        09/15/94 DJP            New
        11/07/94 JMB            Updated argv_t.
        01/14/98 JMB            Added definition of maximum numeric argument type
        05/24/99 JMB ic3.00     Added COMP-5 and POINTER
        04/24/00 JMB ic3.00     Additional ICOBOL 3 updates
        07/07/04 DJP 3.44       Added SQL types to handle varchar in ic_bltn
        04/02/08 DJP 4.00       Added rev2x support
        08/31/10 DJP 4.50       Moved arg_xxx macros fron bltnsubs.h
        12/29/11 DJP 5.00       Removed long modifer from unsigned int
                                which should still yield 32 bits in 64-bit build
                                and swapped the pic_ptr and arg_len for 64-bit
                                aligned addresses in a 64-bit build
*/

#ifndef _CALLDEFS_H_
#define _CALLDEFS_H_

/* The standard builtin function has two arguments and a return value.
   The two arguments are:
        1.  A count of the number of arguments in the argument vector (int).
        2.  A pointer to the argument vector (argv_t *).
    The return value will be passed to the exception status value (status_t).

    The argv entries themselves should never be modified.  The fields named
    'resv' may be non-zero and may contain information used by the runtime
    system.

    The exception values currently used are defined in the supplied header
    file 'status.h'.  If you want to pass back your own exception values
    (e.g. UNIX error codes), we recommend that you do so through a separate
    parameter to your routines,
        e.g. CALL "user-routine" USING arg1, arg2, err-code.

    If there is no error in your routine, you should return the value
    SE_NO_ERR, which has the value 0.

    It is your responsibility to perform any required validation of the
    incoming parameters. If a parameter is not what was expected, you
    should return the error SE_PARM_MISMATCH.  We have made this somewhat
    easier in that the call table contains a minimum and maximum parameter
    count. The runtime will check the incoming parameter count against
    these limits and issue SE_PARM_MISMATCH for you.

    It is your responsibility to be aware of the expected data type and how
    that data type is stored by the runtime system.  In particular,
    COMP items are stored in big-endian (non-byteswapped) format.  If your
    code is running on a byteswapped machine (i.e., Intel 80x86), it is up
    to you to perform the necessary byte swapping.

    NOTE: The picture information for edited arguments is encoded, so there
          is not a way for you to correctly store a value back to this type
          of argument.  We don't recommend that you use edited items as
          arguments.

    NOTE: the runtime builds the argv vector with an additional hidden entry
          that points to the argument representing the name of the called
          routine in the COBOL program.  It occurs immediately before the
          first element of argv.  Thus you copy can argv to a variable of
          type argv_t *, then decrement it to point to the name argument.
          (name_argv = argv - 1;)  Note that this is typically not a
          NUL terminated string, and may even be a literal in the runtime
          system address space, so it should never be modified.
*/

#define MAX_CALL_ARGS       32          /* maximum call arguments */
#define MAX_CALL_NAME_LEN   30          /* maximum length of a name */


/* define data types for builtin arguments */
typedef
unsigned int
    argc_t;                             /* type for count */

typedef
struct {
    union {
        unsigned int
            val;
        struct {
            unsigned char
                type,
                cnt,
                left,
                right;
        } num;                          /* numeric and numeric editied */
        struct {
            unsigned char
                type,
                cnt,                    /* # digits for numeric display */
                resv [2];
        } index;                        /* index */
        struct {
            unsigned char
                type,
                resv [3];
        } alpha;                        /* alphabetic/alphanumeric [justified]/an_ed/group */
        struct {
            unsigned char
                type,
                resv [3];
        } rev2x;                        /* rev2x unknown type */
    } arg_info;
    unsigned int
        arg_len;                        /* Number of bytes in the argument */
    char
        *pic_ptr;                       /* address of picture control string */
    char
        *arg_ptr;                       /* address of the argument data */
} argv_t;


/* the argument type values */
#define ARG_ILL_TYPE            0       /* type not available */
#define ARG_INX_TYPE            1       /* index */
#define ARG_UNS_TYPE            2       /* unsigned display */
#define ARG_TR_OP_TYPE          3       /* trailing overpunch */
#define ARG_TR_SP_TYPE          4       /* trailing separate */
#define ARG_LD_OP_TYPE          5       /* leading overpunch */
#define ARG_LD_SP_TYPE          6       /* leading separate */
#define ARG_UNS_COMP_TYPE       7       /* unsigned comp */
#define ARG_COMP_TYPE           8       /* signed comp */
#define ARG_UNS_COMP3_TYPE      9       /* unsigned comp3 */
#define ARG_COMP3_TYPE          10      /* signed comp3 */
#define ARG_UNS_BIN_TYPE        11      /* unsigned binary */
#define ARG_BIN_TYPE            12      /* signed binary */
#define ARG_UNS_DEC_TYPE        13      /* unsigned packed decimal */
#define ARG_DEC_TYPE            14      /* signed packed decimal */
#define ARG_UNS_INT_TYPE        15      /* unsigned native-order 1/2/4/8 byte integer */
#define ARG_INT_TYPE            16      /* signed native-order 1/2/4/8 byte integer */
#define ARG_FLT4_TYPE           17      /* 4-byte floating point (native format) */
#define ARG_FLT8_TYPE           18      /* 8-byte floating point (native format) */
#define ARG_UNS_COMP5_TYPE      19      /* unsigned comp5 */
#define ARG_COMP5_TYPE          20      /* signed comp5 */
#define ARG_UNS_COMPBIN_TYPE    21      /* unsigned comp/bin combo (SIZE by PIC, move by storage) */
#define ARG_COMPBIN_TYPE        22      /* signed comp/bin combo (SIZE by PIC, move by storage) */
#define ARG_NUM_ED_TYPE         23      /* numeric edited */
#define ARG_MAX_NUM_TYPE        23      /* maximum numeric type */
#define ARG_AB_TYPE             24      /* alphanumeric/alphabetic */
#define ARG_AB_J_TYPE           25      /* alphanumeric/alphabetic Justified */
#define ARG_AN_TYPE             26      /* alphanumeric/alphabetic */
#define ARG_AN_J_TYPE           27      /* alphanumeric/alphabetic Justified */
#define ARG_AN_ED_TYPE          28      /* alphabetic/alphanumeric edited */
#define ARG_GROUP_TYPE          29      /* group */
#define ARG_POINTER_TYPE        30      /* pointer */
#define ARG_CHAR_TYPE           31      /* (sql) character string */
#define ARG_VARCHAR_TYPE        32      /* (sql) varying length character string */
#define ARG_BIT_TYPE            33      /* (sql) bit string */
#define ARG_BIT_VAR_TYPE        34      /* (sql) varying length bit string */
/* NOTE: this ordering must match DTI_xxx ordering */
#define ARG_DTI_BASE            34      /* NOTE: DTI_NULL_TYPE maps here */
#define DTI_TO_ARG(x) ((uint8_t)(ARG_DTI_BASE+(x)))
#define ARG_TO_DTI(x) ((uint8_t)((x)-ARG_DTI_BASE))
#define ARG_DATE_TYPE           35      /* (sql) date */
#define ARG_TIME_TYPE           36      /* (sql) time */
#define ARG_TIMESTAMP_TYPE      37      /* (sql) time-stamp */
#define ARG_YEAR_TYPE           38      /* (sql) year interval */
#define ARG_YEAR_MON_TYPE       39      /* (sql) year-month interval */
#define ARG_MON_TYPE            40      /* (sql) month interval */
#define ARG_DAY_TYPE            41      /* (sql) day interval */
#define ARG_DAY_HOUR_TYPE       42      /* (sql) day-hour interval */
#define ARG_DAY_MIN_TYPE        43      /* (sql) day-minute interval */
#define ARG_DAY_SEC_TYPE        44      /* (sql) day-second interval */
#define ARG_HOUR_TYPE           45      /* (sql) hour interval */
#define ARG_HOUR_MIN_TYPE       46      /* (sql) hour-minute interval */
#define ARG_HOUR_SEC_TYPE       47      /* (sql) hour-second interval */
#define ARG_MIN_TYPE            48      /* (sql) minute interval */
#define ARG_MIN_SEC_TYPE        49      /* (sql) minute-second interval */
#define ARG_SEC_TYPE            50      /* (sql) second interval */
#define ARG_MIN_DTI_TYPE        ARG_DATE_TYPE
#define ARG_MAX_DTI_TYPE        ARG_SEC_TYPE
#define ARG_INDICATOR_TYPE      51      /* (sql) indicator */
#define ARG_REV2X_TYPE          52      /* rev2x runtime arg (unknown type) */
/* the rest are reserved for future use */
#define ARG_RESV53_TYPE         53      /* reserved */
#define ARG_RESV54_TYPE         54      /* reserved */
#define ARG_RESV55_TYPE         55      /* reserved */
#define ARG_RESV56_TYPE         56      /* reserved */
#define ARG_RESV57_TYPE         57      /* reserved */
#define ARG_RESV58_TYPE         58      /* reserved */
#define ARG_RESV59_TYPE         59      /* reserved */
#define ARG_RESV60_TYPE         60      /* reserved */
#define ARG_RESV61_TYPE         61      /* reserved */
#define ARG_RESV62_TYPE         62      /* reserved */
#define ARG_RESV63_TYPE         63      /* reserved */

/* define some macros for type checking the arguments to the builtin functions */

/* check for rev2x type */
#define arg_rev2x(x)    (x.arg_info.alpha.type == ARG_REV2X_TYPE ? TRUE : FALSE)

/* check for all numeric types */
#define arg_numeric(x)  ((x.arg_info.num.type < ARG_MAX_NUM_TYPE || x.arg_info.rev2x.type == ARG_REV2X_TYPE) ? TRUE : FALSE)

/* check for all alpha types - check for varchar as needed in addition */
#define arg_alpha(x)    ((x.arg_info.alpha.type == ARG_AN_TYPE || \
                                x.arg_info.alpha.type == ARG_GROUP_TYPE || \
                                x.arg_info.alpha.type == ARG_AN_J_TYPE || \
                                x.arg_info.alpha.type == ARG_AB_TYPE || \
                                x.arg_info.alpha.type == ARG_AB_J_TYPE || \
                                x.arg_info.alpha.type == ARG_CHAR_TYPE || \
                                x.arg_info.alpha.type == ARG_VARCHAR_TYPE || \
                                x.arg_info.rev2x.type == ARG_REV2X_TYPE) ? TRUE : FALSE)

/* check for varchar */
#define arg_varchar(x)    ((x.arg_info.alpha.type == ARG_VARCHAR_TYPE) ? TRUE : FALSE)

/* Typical builtin call definition:

status_t    user_bltn (argc_t argc, argv_t *argv) {....}

*/

/*  It is your responsibility to build a table of builtin entry points.
    The table is a simple array of call_tbl_t elements.  You initialize
    the table by calling the extern entry point load_user_bltns_ defined
    in this header file.  (The call to load_user_bltns_ is generally made
    from a routine init_user_bltn which you supply in the module you wish
    to add.  See bltnstub.c in the link_kit.)  You supply the base address
    of the table and the count of the number of entries.  You can call the
    load routine more than once for multiple tables.  A table can be unloaded
    by calling the supplied unload_user_bltns_ routine.  It is not necessary
    to unload a table before the runtime terminates.

    If you want to be informed when the runtime system is terminating
    (e.g., to disconnect from a server called by your builtins), you
    must register a termination routine by calling register_user_cleanup_
    with the address of your cleanup routine.  You may register more than
    one routine; they will be called in reverse order.  There is currently
    no way to unregister a cleanup routine.  The register routine will
    return an error only if it cannot allocate memory for the data structure
    used to register your routine.

    NOTE: the runtime always lower-cases names for the builtin lookup, so
          the name strings referenced by your table must be lower-case.
          The link and name length must be initialized to 0.  This is how
          the runtime tells whether or not the entry is loaded.  If it sees
          non-zero values when load_user_bltns_ is called, it will return
          the error SE_PARM_MISMATCH.  The unload routine will reset them
          back to 0.
*/



/* user-defined call entry-point table entry */
typedef
struct call_tbl_s {
    struct call_tbl_s
        *link;                          /* lookup hash link (init to NULL) */
    unsigned int
        name_len,                       /* length of name (init to 0) */
        min_args,                       /* minimum number of args */
        max_args;                       /* maximum number of args */
    status_t
        (*fcn) (argc_t argc, argv_t *argv);
    const char
        *name;                          /* static/literal lowercase name string */
} call_tbl_t;

extern  call_tbl_t *lookup_user_call_ (const char *name, unsigned int name_len);
extern  status_t    load_user_calls_ (unsigned int count, call_tbl_t call_tbl []);
extern  void        unload_user_calls_ (unsigned int count, call_tbl_t call_tbl []);
extern  status_t    register_user_cleanup_ (void (*fcn) (void));

#ifndef NULL
#define NULL    0
#endif

#endif

