/*  status_t.h

    Copyright 1991, 1992, 1994, 1995, 2006 Envyr Corporation.
    All rights reserved.

    Revision History:
        ichost 3.20
            04/16/91    LAD     New.
        ichost 3.30
            12/04/91    LAD     Syncronized with DOS ICHOST 2.XX
            02/20/92    DJP     New messages
            02/24/92    LAD     Matched up with syserrs.
        sys2.10
            08/03/94    JMB     Removes syserrs duplicate (build now appends
                                syserrs to this file for shipment).  Renamed
                                status_t.h
            10/21/94    JMB     Enabled OS error in status codes on UNIX/VS.
        sys2.12
            08/25/95    LAD     General clean up eliminating 16-bit definitions.
        10/17/06 DJP 3.60       Merged group and message number fields,
                                changed icmakems to directly use .h source format.
*/

/* standard error handling definitions
    A zero return values means no error occurred
    A positive return value means an error occurred
    A negative return value means the operation was ok, but a status
        code is being returned.
    The success and failure macros can be used to test function results.
    The status code itself is divided into an operating system error,
        severity level, and message number fields, which may be
        extracted with the get_oserr, get_sevnum, and get_errnum
        macros respectively.
    Note that get_errnum must be used to compare a status code to any
        particular error code defined below, otherwise values in the other
        fields may cause the comparison to inappropriately fail.
*/

#ifndef _STATUS_T_H_
#define _STATUS_T_H_

typedef int status_t;

/* define info bit */
#define INFOVAL_BIT     0x80000000      /* set sign bit for info */
#define INFOVAL_MASK    0x7FFFFFFF      /* mask all but info indicator */

#define SUCCESS         ((status_t) 0)  /* no error */

/* macros to test results */
#define success(a)      ((a)<=0)
#define failure(a)      ((a)>0)
#define has_info(a)     ((a)<0)

/* define OS error part */
#define OSERR_SHIFT     16              /* shift value for os error */
#define OSERR_MASK      0x7FFF0000      /* mask for shifted value */

/* define severity level info */
#define SEV_SHIFT       12              /* shift value for severity */
#define MAX_SEV_COUNT   8               /* current max number of severities */
#define MAX_SEV_SIZE    10              /* max size of msg text (w/ nul) */
#define MAX_SEV_LEN     9               /* max size of msg (w/o nul) */
#define SEV_MASK        ((status_t)7<<SEV_SHIFT)    /* mask for message severity level */
#define SEV_NONE        ((status_t)0<<SEV_SHIFT)    /* no severity preamble */
#define SEV_INFO        ((status_t)1<<SEV_SHIFT)    /* 'Info:    ' preamble */
#define SEV_WARNING     ((status_t)2<<SEV_SHIFT)    /* 'Warning: ' preamble */
#define SEV_ERROR       ((status_t)3<<SEV_SHIFT)    /* 'Error:   ' preamble */
#define SEV_FATAL       ((status_t)4<<SEV_SHIFT)    /* 'Fatal:   ' preamble */
#define SEV_PANIC       ((status_t)5<<SEV_SHIFT)    /* 'Panic:   ' preamble */
#define SEV_RESV1       ((status_t)6<<SEV_SHIFT)    /* reserved */
#define SEV_SPACES      ((status_t)7<<SEV_SHIFT)    /* '         ' preamble */
#define get_sevnum(s)   ((unsigned int)(((s)&SEV_MASK)>>SEV_SHIFT))

/* maximum of 4096 messages */
#define MSG_MASK        ((status_t)4095)    /* mask value for message number */
#define MAX_MSG_COUNT   4096                /* maximum message count per group */
#define MAX_MSG_SIZE    128                 /* max size of msg text (w/ nul) */
#define MAX_MSG_LEN     127                 /* max size of msg (w/o nul) */
#define MAX_MSG_BUF     (MAX_MSG_SIZE+MAX_SEV_SIZE+OSERR_MAX_LEN)
                                            /* max size required for buffer */

/* macros to extract pieces of a status code as integer values */
#define get_oserr(s)    ((unsigned int)(((s)&OSERR_MASK)>>OSERR_SHIFT))
#define set_oserr(s,e)  ((status_t)(((s)&~OSERR_MASK)|((status_t)(e)<<OSERR_SHIFT)))
#define get_errnum(s)   ((unsigned int)((s)&(MSG_MASK)))

/* macro to substitute a different error code */
#define put_errnum(s,e) ((status_t)(((s)&~(MSG_MASK))|(e)))

/* macros to manipulate the info value and indicator bit */
#define get_info(s)     ((unsigned int)((s)&(MSG_MASK)))
#define put_info(s,e)   ((status_t)(((s)&~(MSG_MASK))|(INFOVAL_BIT|((e)&(MSG_MASK)))))
#define clear_info(s)   ((status_t)(((s)&INFOVAL_MASK)))
#define set_info(s)     ((status_t)(((s)|INFOVAL_BIT)))

#endif

